from rest_framework import permissions


# Permissions mixins

class OwnerPermission(permissions.BasePermission):

    message = 'You must be the owner.'

    def has_object_permission(self, request, view, obj):
        if (request.user == obj):
            return True
        return False

class StaffPermission(permissions.BasePermission):
    message = 'You must be a staff member.'

    def has_permission(self, request, view):
        if request.user.is_staff:
            return True
        return False

class OwnerOrStaffPermission(permissions.BasePermission):

    message = 'You must be the owner or a staff member.'

    def has_object_permission(self, request, view, obj):
        if (request.user == obj) or request.user.is_staff:
            return True
        return False



import logging
import uuid

from django.conf import settings
from django.contrib.auth.hashers import identify_hasher, get_hasher
from django.contrib.auth.models import AbstractUser
from django.db import models
from django.db import transaction
from django.db.models import F
from django.db.models.signals import pre_delete, pre_save, post_save
from django.dispatch import receiver
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse

from ..api_gateway.functions import (create_consumer, delete_consumer,
                                        create_consumer_jwt,
                                        create_consumer_acl)
from ..devices.models import Device
from ..services.models import Service, Subscription
from ..utils import get_random_string

from ...models import CreatedAtModel, UpdatableModel, DeactivableModel
from ...common import PROFILE_FIELDS

logger = logging.getLogger('users')


class AbstractUserModel(CreatedAtModel, UpdatableModel, DeactivableModel):
    """
    Abstract user model, the fields below should be defined by 'django.contrib.auth.models.AbstractUser'

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    """

    class Meta:
        abstract = True

    sso_id = models.UUIDField(unique=True, db_index=True, default=uuid.uuid4, editable=False)
    sso_rev = models.PositiveIntegerField(default=1)

    apigw_consumer_id = models.CharField(max_length=36, null=True, blank=True)

    role = models.SmallIntegerField(null=True, blank=True)

    description = models.TextField(_('description'), null=True, blank=True)
    picture = models.TextField(_('picture'), null=True, blank=True)
    birthdate = models.DateField(_('birthdate'), null=True, blank=True)

    latitude = models.FloatField(_('latitude'), null=True, blank=True)
    longitude = models.FloatField(_('longitude'), null=True, blank=True)

    country = models.CharField(_('country'), max_length=46, null=True,
                               blank=True)

    address = models.TextField(_('address'), null=True, blank=True)
    language = models.CharField(_('language'), null=True, blank=True)

    unsubscribed_at = models.DateTimeField(null=True, blank=True)

    language = models.CharField(max_length=3, null=True, blank=True)

    @property
    def is_unsubscribed(self):
        return self.unsubscribed_at is not None

    @is_unsubscribed.setter
    def is_unsubscribed(self, value):
        self.unsubscribed_at = timezone.now()

    def __str__(self):
        return '{}:{}'.format(self.username, self.id)

    def serialized_as_string(self):
        serialized = ''
        for f in PROFILE_FIELDS:
            field = getattr(self, f, None)
            if f == 'password':
                pass
                """
                if field and len(field):
                    print('PASS', f, field)
                    (algorithm, iterations, salt, hash) = field.split('$')
                    print('PASS', (algorithm, iterations, salt, hash))
                    serialized += hash
                else:
                    serialized += str(field)
                """
            else:
                serialized += str(field)


        # logger.debug('serialized_as_string {}'.format(serialized))
        return serialized

    def __init__(self, *args, **kwargs):
        super(AbstractUserModel, self).__init__(*args, **kwargs)
        setattr(self, '__serialized_as_string', self.serialized_as_string())

    def save(self, *args, **kwargs):
        if self._state.adding:
            if not self.first_name:
                self.first_name = ''
            if not self.last_name:
                self.last_name = ''
        return super(AbstractUserModel, self).save(*args, **kwargs)

    def check_password(self, raw_password):
        # password is hashed, checking if will be runtime hardened
        # taken from django.contrib.auth.hashers.check_password
        old_hashed_password = self.password

        preferred = get_hasher('default')
        hasher = identify_hasher(old_hashed_password)

        hasher_changed = hasher.algorithm != preferred.algorithm

        if hasher_changed:
            logger.info(
                '!!! Password hasher for {} changed from {} to {}'.format(self,
                                                                          hasher.algorithm,
                                                                          preferred.algorithm))

        must_update = hasher_changed or hasher.must_update(old_hashed_password)

        if must_update:
            setattr(self, '__password_hardened', True)

        return super(AbstractUserModel, self).check_password(raw_password)

    def create_apigw_consumer(self):
        logger.info('Adding apigw consumer for User {0}'.format(self))
        if not settings.TESTING_MODE:
            status_code_1, consumer = create_consumer(self)
            if status_code_1 != 201:
                logger.error('Error ({0}) creating apigw consumer, {1}'.format(
                    status_code_1, consumer))
                raise Exception(
                    "Error ({0}) creating apigw consumer, {1}".format(
                        status_code_1, consumer))

            status_code_2, acl = create_consumer_acl(self)
            if status_code_2 != 201:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) creating apigw consumer acl, {1}'.format(
                        status_code_2, acl))
                raise Exception(
                    "Error {0} creating apigw consumer acl, {1}".format(
                        status_code_2, acl))
            self.apigw_consumer_id = consumer['id']

        else:
            self.apigw_consumer_id = get_random_string(36)

        self.save()

    def add_user_device(self, fingerprint, user_agent=None):
        logger.info(
            'Adding User Device for user {0} with fingerprint {1}'.format(self,
                                                                          fingerprint))

        device = Device(user=self, fingerprint=fingerprint)

        if not settings.TESTING_MODE:
            status_code, r1 = create_consumer_jwt(self)
            if status_code != 201:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) creating apigw consumer JWT, {1}'.format(
                        status_code, r1))
                raise Exception(
                    "Error creating apigw consumer jwt, {}".format(status_code))

            device.apigw_jwt_id = r1.get('id')
            device.apigw_jwt_key = r1.get('key')
            device.apigw_jwt_secret = r1.get('secret')

        else:
            device.apigw_jwt_id = get_random_string(36)
            device.apigw_jwt_key = get_random_string(32)
            device.apigw_jwt_secret = get_random_string(32)

        device.save()

        return device

    def remove_user_device(self, device):
        logger.info('Removing Device {0}'.format(device.id))
        device.delete()
        return 1

    def remove_all_user_devices(self):
        logger.info('Removing All User Devices for {0}'.format(self))
        removed = 0
        for d in self.devices.all():
            removed += self.remove_user_device(d)
        return removed

    def update_rev(self, commit=False):
        logger.info('Updating rev for User {0}'.format(self))
        self.sso_rev = F('sso_rev') + 1
        if commit:
            self.save()
            self.refresh_from_db()

    def unsubscribe(self):
        logger.info('Unsubscribing User {0}'.format(self))
        try:
            with transaction.atomic():
                self.unsubscribed_at = timezone.now()
                self.is_active = False
                self.update_rev(True)

        except Exception as e:
            logger.error(
                'Unsubscription error {0} for User {1}'.format(e, self),
                exc_info=True)

    def subscribe_to_service(self, referrer, update_rev=False, commit=True):
        logger.info('Subscribinig {0} to service {1}'.format(self, referrer))

        service = Service.objects.get(url=referrer)

        subscription, _subscription_created = \
            Subscription.objects.get_or_create(
            user=self, service=service)

        if _subscription_created:
            logger.info('Created service subscrption for {0}'.format(self))
            if update_rev:
                setattr(self, '__subscription_updated', True)
                self.update_rev(commit)
        else:
            logger.info(
                'User {0} already subscribed to {1}'.format(self, service))

        return _subscription_created

    def unsubscribe_from_service(self, subscription, update_rev=False,
                                 commit=True):
        logger.info('Unubscribinig {0} form service {1}'.format(self,
                                                                subscription.service))
        setattr(self, '__subscription_updated', True)

        subscription.unsubscribed_at = timezone.now()
        subscription.save()

        if update_rev:
            self.update_rev(commit)


class User(AbstractUser, AbstractUserModel):
    """
    User class
    """

    def get_absolute_url(self):
        return reverse("user:detail", args=[self.sso_id])


# Signals

@receiver(pre_save, sender=User)
def check_upated_fields(sender, instance, **kwargs):
    if not instance._state.adding:  # if instance.pk:
        email_updated = hasattr(instance, '__email_updated')
        subscription_updated = hasattr(instance, '__subscription_updated') or instance.is_unsubscribed
        password_updated = hasattr(instance, '__password_updated')
        field_updated = instance.__serialized_as_string != \
                        instance.serialized_as_string()

        update_rev = False

        if field_updated:
            logger.info('User {0} updated PROFILE_FIELDS \n{1}\n{2}'.format(instance, instance.__serialized_as_string, instance.serialized_as_string()))
            update_rev = True

        if email_updated:
            logger.info('User {0} updated email'.format(instance))

        if password_updated:
            logger.info('User {0} updated password'.format(instance))

        if subscription_updated:
            logger.info('User {0} updated subscription'.format(instance))

        if email_updated or password_updated or field_updated or subscription_updated:
            # __password_hardened attribute set by
            # backend.serializers.LoginSerializer on runtime password hardening
            update_rev = True

        if hasattr(instance, '__password_hardened'):
            update_rev = False

        if update_rev:
            instance.update_rev()  # updating rev
            setattr(instance, '__rev_updated', True)

            logger.info('rev updated, removing all devices for user {}.'.format(instance))
            instance.remove_all_user_devices()


@receiver(post_save, sender=User)
def fetch_from_db_if_updated(sender, instance, created, **kwargs):
    if not created and hasattr(instance, '__rev_updated'):
        logger.info('User {0} updated rev, refreshing instance from DB'.format(instance))
        instance.refresh_from_db()


@receiver(pre_delete, sender=User)
def delete_apigw_user_consumer(sender, instance, **kwargs):
    logger.info('Deleting apigw Consumer for User {0}'.format(instance))
    if not settings.TESTING_MODE:
        status_code, r1 = delete_consumer(instance)
        logger.info('Kong consumer deleted ({0}), {1}'.format(status_code, r1))
        if status_code >= 300:
            if status_code != 404:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) Deleting apigw consumer for User {1}, {2}'.format(
                        status_code, instance, r1))
                raise Exception(
                    "Error deleting apigw consumer, {}".format(status_code))



from django.conf import settings

if settings.DJANGO_SSO_BACKEND_ENABLED:
    import json

    import jwt
    from allauth.account.models import EmailAddress
    from django.contrib.auth.hashers import make_password
    from django.test import TestCase, Client
    from django.urls import reverse
    from rest_framework import status

    from ...services.models import Service, Subscription
    from ..models import User


    class UserTestCase(TestCase):
        def setUp(self):
            self.client = Client()

            email = self.email = "pippo@disney.com"
            username = self.username = "pippo"
            password = self.password = "paperina"
            updated_password = "pippapippa"


            username2 = "cenerentola"
            email2 = "cenerentola@disney.com"
            password2 = "tanticassi"

            self.user = User.objects.create_user(username=username, email=email, password=password)
            self.user_email = EmailAddress.objects.create(user=self.user,
                                                          email=email,
                                                          primary=True,
                                                          verified=True)

            self.user2 = User.objects.create_user(username=username2, email=email2, password=password2)
            self.user2_email = EmailAddress.objects.create(user=self.user2,
                                                          email=email2,
                                                          primary=True,
                                                          verified=True)

            self.service = Service.objects.create(url='http://example.com')
            self.user_subscription = Subscription.objects.create(user=self.user,
                                                                 service=self.service)
            self.service2 = Service.objects.create(url='http://disney.org')

            self.staff_user_username = 'staff'
            self.staff_user_password = 'abc123456'
            self.staff_user = User.objects.create_user(username=self.staff_user_username, email='staff@example.com', password=self.staff_user_password, is_staff=True)
            self.staff_user_email = EmailAddress.objects.create(user=self.staff_user,
                                                          email=self.staff_user.email,
                                                          primary=True,
                                                          verified=True)

            self.valid_new_profile = {
                'email': 'paperino@gmail.com',
                'username': 'paperino',
                'password': 'nonnapapera',
                'first_name': 'Paolino',
                'referrer': 'http://example.com'
            }

            self.valid_new_profile_hashed_pw = {
                'email': 'paperino@gmail.com',
                'username': 'paperino',
                'password': make_password('nonnapapera'),
                'first_name': 'Paolino',
                'referrer': 'http://example.com'
            }

            self.valid_new_profile_login = {
                'username': 'paperino',
                'password': 'nonnapapera',
                'fingerprint': '123456'
            }


            self.staff_user_valid_login = {
                'username': self.staff_user.username,
                'password': self.staff_user_password,
                'fingerprint': '123456'
            }

            self.valid_login = {
                'username': username,
                'password': password,
                'fingerprint': '123456'
            }

            self.valid_staff_login = {
                'username': 'staff',
                'password': self.staff_user_password,
                'fingerprint': '123456'
            }

            self.valid_unsubscription = {
                'password': password
            }

            self.valid_login_after_password_change = {
                'username': username,
                'password': updated_password,
                'fingerprint': '123456'
            }

            self.valid_profile_update = {
                'first_name': 'Pippo'
            }

            self.valid_password_update = {
                'password': updated_password
            }

            self.valid_email_update = {
                'email': 'pippo2@gmail.com'
            }

            self.device0 = self.user.add_user_device('000000')
            self.device1 = self.user.add_user_device('111111')

            self.valid_registration = {
                'email': 'topolino@disney.com',
                'username': 'topolino',
                'password1': 'paperina',
                'password2': 'paperina',
                'first_name': 'Mickey',
                'last_name': 'Mouse',
                'country': 'Italia',
                'latitude': 12.345,
                'longitude': 54.321,
                'referrer': 'http://example.com'
            }



    class TestUserCreation(UserTestCase):

        def test_user_registration_creates_service_subscription(self):
            """
              Registration via rest_auth view
            """
            response = self.client.post(
                reverse('rest_register'),
                data = json.dumps(self.valid_registration),
                content_type='application/json'
            )
            print('RESPONSE', response.status_code, response.data)

            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

            new_user = User.objects.filter(email=self.valid_registration['email']).first()
            subscription = Subscription.objects.filter(user=new_user).first()

            self.assertNotEqual(subscription, None)


        def test_user_creation_by_staff_creates_new_user(self):
            """
            User created by staff param can NOT login witouth confirmating email
            """
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            response2 = self.client.post(
                '/api/v1/users/',
                data=json.dumps(self.valid_new_profile),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.assertEqual(response2.status_code, 201, 'can not create user')

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            created_user = User.objects.filter(username=self.valid_new_profile.get('username')).first()

            #print('\n\n\n\nRESP\n', response2.data, '\n\n\n\nUSERS:', EmailAddress.objects.filter(user=created_user))

            self.assertNotEqual(created_user, None, 'new user not created')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_new_profile_login),
                content_type='application/json'
            )
            cookies3 = response3.cookies

            self.assertEqual(response3.status_code, 400, 'user can login without confirmating email change')

            self.assertEqual(response3.data.get('non_field_errors').pop(), 'E-mail is not verified.')

            subscription = Subscription.objects.filter(user=created_user).first()

            self.assertNotEqual(subscription, None, 'subscription NOT created')


        def test_user_creation_by_staff_with_hashed_password_creates_new_user(self):
            """
            User created by staff param can NOT login witouth confirmating email
            """
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            response2 = self.client.post(
                '/api/v1/users/?password_is_hashed=true',
                data=json.dumps(self.valid_new_profile_hashed_pw),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.assertEqual(response2.status_code, 201, 'can not create user')

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            created_user = User.objects.filter(username=self.valid_new_profile.get('username')).first()

            #print('\n\n\n\nRESP\n', response2.data, '\n\n\n\nUSERS:', EmailAddress.objects.filter(user=created_user))

            self.assertNotEqual(created_user, None, 'new user not created')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_new_profile_login),
                content_type='application/json'
            )
            cookies3 = response3.cookies

            self.assertEqual(response3.status_code, 400, 'user can login without confirmating email change')

            self.assertEqual(response3.data.get('non_field_errors').pop(), 'E-mail is not verified.')


        def test_user_creation_by_staff_creates_confirmed_new_user(self):
            """
            User created by staff with skip_confirmation query param can login witouth confirmating email
            """
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            response2 = self.client.post(
                '/api/v1/users/' + '?skip_confirmation=true',
                data=json.dumps(self.valid_new_profile),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.assertEqual(response2.status_code, 201, 'can not create user')

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            created_user = User.objects.filter(username=self.valid_new_profile.get('username')).first()

            print('\n\n\n\nRESP X\n', response2.data, '\n\n\n\nUSERS:', EmailAddress.objects.filter(user=created_user))

            self.assertNotEqual(created_user, None, 'new user not created')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_new_profile_login),
                content_type='application/json'
            )
            cookies3 = response3.cookies

            # print('\n\n\n\nRISPOSTA3', response3.data)

            self.assertEqual(response3.status_code, 200, 'user can NOT login without confirmating email change')


        def test_normal_user_cannot_create_user(self):
            """
            Normal user cannot create users
            """
            first_rev = self.user.sso_rev
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies = response.cookies

            response2 = self.client.post(
                '/api/v1/users/',
                data=json.dumps(self.valid_new_profile),
                content_type='application/json'
            )
            cookies2 = response2.cookies
            print('\n RISPOSTA', response2.data)
            created_user = User.objects.filter(email=self.valid_login.get('email')).first()

            self.assertEqual(created_user, None, 'new user created by user')

            self.assertEqual(response2.status_code, 403, 'User can create user!')



    class TestUserRetrieve(UserTestCase):

        def test_user_can_retrieve_profile_with_subscriptions(self):
            """
              Registration via rest_auth view
            """
            print('FIRST', json.dumps(self.valid_login))

            response = self.client.post(
                reverse('rest_login'),
                data=json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies = response.cookies
            print('RISPSTA1', response.data)

            self.assertEqual(response.status_code, status.HTTP_200_OK)

            response2 = self.client.get(
                reverse('users:user_detail', args=(self.user.sso_id, )),
                content_type='application/json'
            )

            self.assertEqual(response2.status_code, status.HTTP_200_OK)

            print('RISPSTA2', response2.data)

            self.assertEqual(response2.data.get('subscriptions'), [{'url': el.service.url, 'is_unsubscribed': el.is_unsubscribed} for el in self.user.subscriptions.all()])



    class TestUserUpdate(UserTestCase):

        def test_user_update_refreshes_and_returns_current_device_jwt_and_deletes_other_devices(self):
            """
            User update refreshes caller jwt and deletes all but caller devices
            """
            first_rev = self.user.sso_rev
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies = response.cookies
            #print('\n\n\nRESPONSE1!!!!!!\n\n', response.data)

            response2 = self.client.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_profile_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            #print('\n\n\nRESPONSE!!!!!!\n\n', response2.data)

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            self.assertTrue(len(cookies2.keys()) > 0, 'no jwt cookie set')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 1, 'other devices not deleted')

            self.assertEqual(devices.first().fingerprint, self.valid_login['fingerprint'], 'device not updated')

            jwt_cookie = cookies.get('jwt').value
            jwt_cookie2 = cookies2.get('jwt').value

            # print('\nRISPOSTA, user rev', self.user.sso_rev, '\n\nTOKEN', jwt_cookie)

            unverified_payload = jwt.decode(jwt_cookie, None, False)
            unverified_payload2 = jwt.decode(jwt_cookie2, None, False)

            self.assertEqual(unverified_payload2.get('rev'), unverified_payload.get('rev')+1, 'rev not incremented in cookie')

        def test_user_email_update_deletes_all_devices(self):
            """
            User update refreshes caller jwt and deletes all but caller devices
            """
            print('\n logging in with user', self.user, 'and fingerprint', self.valid_login.get('fingerprint'))
            first_rev = self.user.sso_rev
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies = response.cookies

            print('\n sending patch from user', self.user, 'with new email', self.valid_email_update)
            response2 = self.client.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_email_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 0, 'devices not deleted')

        def test_user_password_update_refreshes_and_returns_current_device_jwt_and_deletes_all_devices_but_the_caller(self):
            """
            User password update refreshes caller jwt and deletes all but caller devices
            """

            first_rev = self.user.sso_rev
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies = response.cookies
            self.assertEqual(response.status_code, 200, 'unable to login with valid credentials')

            response2 = self.client.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_password_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            self.assertTrue(len(cookies2.keys()) > 0, 'no jwt cookie set')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 1, 'other devices not deleted')

            self.assertEqual(devices.first().fingerprint, self.valid_login['fingerprint'], 'device not updated')

            jwt_cookie = cookies.get('jwt').value
            jwt_cookie2 = cookies2.get('jwt').value

            # print('\nRISPOSTA, user rev', self.user.sso_rev, '\n\nTOKEN', jwt_cookie)

            unverified_payload = jwt.decode(jwt_cookie, None, False)
            unverified_payload2 = jwt.decode(jwt_cookie2, None, False)

            self.assertEqual(unverified_payload2.get('rev'), unverified_payload.get('rev')+1, 'rev not incremented in cookie')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login_after_password_change),
                content_type='application/json'
            )
            #print('\n\n\n RISPOSTS!', response3.data)
            cookies3 = response3.cookies

            self.assertEqual(response3.status_code, 200, 'unable to login with new password')
            jwt_cookie3 = cookies3.get('jwt').value
            unverified_payload3 = jwt.decode(jwt_cookie3, None, False)

            print('\n\n\n Unv pay', unverified_payload3)

            self.assertEqual(unverified_payload2.get('rev'), unverified_payload3.get('rev'), 'rev changed after login')

        def test_user_password_update_refreshes_profile_if_called_with_authorization_headers(self):
            """
            User password update
            """

            first_rev = self.user.sso_rev
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            print('RESPONSE1', response.data)
            cookies = response.cookies
            self.assertEqual(response.status_code, 200, 'unable to login with valid credentials')

            received_jwt = cookies.get('jwt').value
            client2 = Client()

            response2 = client2.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_password_update),
                content_type='application/json',
                HTTP_AUTHORIZATION='Bearer {}'.format(received_jwt)
            )

            print('RESPONSE2', self.valid_password_update, response2.data)

            self.assertEqual(response2.status_code, 200, 'cannot patch with jwt inside headers')
            cookies2 = response2.cookies

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            self.assertTrue(len(cookies2.keys()) > 0, 'no jwt cookie set')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 1, 'other devices not deleted')

            self.assertEqual(devices.first().fingerprint, self.valid_login['fingerprint'], 'device not updated')

            jwt_cookie = cookies.get('jwt').value
            jwt_cookie2 = cookies2.get('jwt').value

            # print('\nRISPOSTA, user rev', self.user.sso_rev, '\n\nTOKEN', jwt_cookie)

            unverified_payload = jwt.decode(jwt_cookie, None, False)
            unverified_payload2 = jwt.decode(jwt_cookie2, None, False)

            self.assertEqual(unverified_payload2.get('rev'), unverified_payload.get('rev')+1, 'rev not incremented in cookie')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login_after_password_change),
                content_type='application/json'
            )
            #print('\n\n\n RISPOSTS!', response3.data)
            cookies3 = response3.cookies

            self.assertEqual(response3.status_code, 200, 'unable to login with new password')
            jwt_cookie3 = cookies3.get('jwt').value
            unverified_payload3 = jwt.decode(jwt_cookie3, None, False)

            print('\n\n\n Unv pay', unverified_payload3)

            self.assertEqual(unverified_payload2.get('rev'), unverified_payload3.get('rev'), 'rev changed after login')

        def test_user_update_by_staff_deletes_all_user_devices(self):
            """
            User update by staff refreshes deletes all user devices
            """
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            first_rev = self.user.sso_rev
            response2 = self.client.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_profile_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 0, 'user devices not deleted')

        def test_user_email_update_by_staff_disables_user_login_user_must_confirm_email(self):
            """
            User update by staff refreshes disables login and deletes all user devices
            """
            first_rev = self.user.sso_rev
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            response2 = self.client.patch(
                '/api/v1/users/{0}/'.format(self.user.sso_id),
                data = json.dumps(self.valid_email_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 0, 'user devices not deleted')

            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies3 = response3.cookies

            self.assertEqual(response3.status_code, 400, 'user can login without confirmating email change')

            self.assertEqual(response3.data.get('non_field_errors').pop(), 'E-mail is not verified.')


        def test_user_email_update_by_staff_with_skip_email_confirmation_activates_new_email(self):
            """
            User update by staff refreshes disables login and deletes all user devices
            """
            first_rev = self.user.sso_rev
            self.client.login(username=self.staff_user_username, password=self.staff_user_password)

            print('\n sending patch from user', self.staff_user, 'to user', self.user, 'with new email', self.valid_email_update)
            response2 = self.client.patch(
                '/api/v1/users/{0}/?skip_confirmation=true'.format(self.user.sso_id),
                data = json.dumps(self.valid_email_update),
                content_type='application/json'
            )
            cookies2 = response2.cookies
            print('Patch response', response2.status_code)

            self.assertTrue(len(cookies2.keys()) == 0, 'jwt cookie set')

            self.user.refresh_from_db()
            second_rev = self.user.sso_rev

            self.assertEqual(second_rev, first_rev+1, 'rev not incremented')

            devices = self.user.devices.all()

            self.assertEqual(devices.count(), 0, 'user devices not deleted')

            print('\n Trying to login from user', self.user, 'with credentials', self.valid_login)
            response3 = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_login),
                content_type='application/json'
            )
            cookies3 = response3.cookies

            print('\n Login from user', self.user, 'returns', response3.data)

            self.assertEqual(response3.status_code, 200, 'user can NOT login without confirmating email change')




import logging
from django.contrib.auth import get_user_model, forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.forms import EmailField

from allauth.utils import email_address_exists
from allauth.account.models import EmailAddress

logger = logging.getLogger('users')
User = get_user_model()


class UserChangeForm(forms.UserChangeForm):
    class Meta(forms.UserChangeForm.Meta):
        model = User


class UserCreationForm(forms.UserCreationForm):

    error_message = forms.UserCreationForm.error_messages.update(
        {"duplicate_username": _("This username has already been taken.")}
    )

    email = EmailField(required=True, label='Email')

    class Meta(forms.UserCreationForm.Meta):
        model = User
        fields = ("username", "email",)

    def clean_email(self):
        email = self.cleaned_data["email"]

        if email_address_exists(email):
            raise ValidationError(self.error_messages["duplicate_email"])
        else:
            return email

    def clean_username(self):
        username = self.cleaned_data["username"]

        try:
            User.objects.get(username=username)
        except User.DoesNotExist:
            return username

        raise ValidationError(self.error_messages["duplicate_username"])

    def __init__(self, *args, **kwargs):
        super(UserCreationForm, self).__init__(*args, **kwargs)

        if settings.DJANGO_SSO_BACKEND_ENABLED:
            # removing unnecessary password fields
            self.fields['password1'].required = False
            self.fields['password2'].required = False
            # If one field gets autocompleted but not the other, our 'neither
            # password or both password' validation will be triggered.
            self.fields['password1'].widget.attrs['autocomplete'] = 'off'
            self.fields['password2'].widget.attrs['autocomplete'] = 'off'

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit)

        if not commit:
            new_email_address = EmailAddress.objects.create(
                user=user,
                email=user.email,
                verified=True,
                primary=True
            )
            logger.info('NEW USER EMAIL CREATED FROM ADMIN: {}'.format(new_email_address))

        return user



import logging

import jwt

from .models import Device

from django_sso_app.backend.functions import get_request_jwt

logger = logging.getLogger('devices')


def delete_request_device(request):
    removing_device = get_request_device(request)
    if removing_device is not None:
        logger.info('Removing logged out user device {0} for User {1}'.format(removing_device, request.user))
        request.user.remove_user_device(removing_device)
    else:
        logger.warning('User {0} is logging out WITHOUT removing device.'.format(request.user))


def get_request_device(request):
    received_jwt = get_request_jwt_fingerprint(request)
    return Device.objects.filter(fingerprint=received_jwt).first()


def get_request_jwt_fingerprint(request):
    received_jwt = get_request_jwt(request)
    if received_jwt is None:
        raise KeyError('No token specified')

    unverified_payload = jwt.decode(received_jwt, None, False)
    return unverified_payload.get('fp')



from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.i18n import JavaScriptCatalog
from django.utils import timezone
from django.views.decorators.http import last_modified
from django.views.static import serve
from django.views.generic import TemplateView

from django.contrib.auth.decorators import login_required

from rest_framework_swagger.views import get_swagger_view

from django_sso_app.core.apps.groups.urls import urlpatterns as groups_urls

if settings.DJANGO_SSO_BACKEND_ENABLED:
    from .views import (ProfileView, ProfileUpdateView,
                        LoginAPIView, LogoutAPIView,
                        UserDetailsAPIView,
                        PasswordResetAPIView, PasswordResetConfirmAPIView, PasswordChangeAPIView,
                        RegisterAPIView, VerifyEmailAPIView,
                        LoginView, LogoutView, SignupView,
                        AccountInactiveView,
                        EmailView, ConfirmEmailView, EmailVerificationSentView,
                        PasswordResetFromKeyView, PasswordResetFromKeyDoneView,
                        PasswordResetDoneView, PasswordResetView,
                        PasswordChangeView, PasswordSetView,
                        WebpackBuiltTemplateView)

last_modified_date = timezone.now()

js_info_dict = {}

urlpatterns = [
    # sso
    #url(r'^$', WebpackBuiltTemplateView.as_view(), name='home'),

    # django views
    url(r'^admin/', admin.site.urls),

    url(r'^docs/$', get_swagger_view(title='API Docs'), name='api_docs'),

    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^jsi18n/$', last_modified(lambda req, **kw: last_modified_date)(JavaScriptCatalog.as_view()),
        js_info_dict,
        name='javascript-catalog'),
]

if settings.DJANGO_SSO_BACKEND_ENABLED:
    urlpatterns += [
        url(r'^$', ProfileView.as_view(), name='sso_profile'),

        url(r'^profile/update/$', ProfileUpdateView.as_view(), name='sso_profile_update'),

        url(r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
            WebpackBuiltTemplateView.as_view(),
            name='restauth_password_reset_confirm'),

        # rest_auth
        url(r'^api/v1/auth/registration/$', RegisterAPIView.as_view(), name='rest_register'),
        url(r'^api/v1/auth/verify-email/$', VerifyEmailAPIView.as_view(), name='rest_verify_email'),
        # completely overrided
        # url(r'^api/v1/auth/registration/', include('rest_auth.registration.urls')),

        # rest_auth
        url(r'^api/v1/auth/login/', LoginAPIView.as_view(), name='rest_login'),
        url(r'^api/v1/auth/logout/', LogoutAPIView.as_view(), name='rest_logout'),

        url(r'^api/v1/auth/user/', UserDetailsAPIView.as_view(), name='user_helper_view'),

        url(r'^api/v1/auth/password/reset/$', PasswordResetAPIView.as_view(), name='rest_password_reset'),
        url(r'^api/v1/auth/password/reset/confirm/$', PasswordResetConfirmAPIView.as_view(),
            name='rest_password_reset_confirm'),
        url(r'^api/v1/auth/password/change/$', PasswordChangeAPIView.as_view(),
            name='rest_password_change'),
        # completely overrided
        # url(r'^api/v1/auth/', include('rest_auth.urls')),

        # url(r'^api/v1/groups/', include('django_sso_app.core.apps.groups.urls', namespace='groups')),
        # url(r'^api/v1/profiles/', include('django_sso_app.core.apps.profiles.urls', namespace='profiles')),
        # url(r'^api/v1/services/', include('django_sso_app.core.apps.services.urls', namespace='services')),
        # url(r'^api/v1/passepartout/', include('django_sso_app.core.apps.passepartout.urls', namespace='passepartout')),

        # allauth
        url(r"^signup/$", SignupView.as_view(), name="account_signup"),
        url(r"^login/$", LoginView.as_view(), name="account_login"),
        url(r"^logout/$", LogoutView.as_view(), name="account_logout"),

        url(r"^password/change/$", login_required(PasswordChangeView.as_view()),
            name="account_change_password"),
        url(r"^password/set/$", login_required(PasswordSetView.as_view()), name="account_set_password"),

        url(r"^inactive/$", AccountInactiveView.as_view(), name="account_inactive"),

        # E-mail
        url(r"^email/$", login_required(EmailView.as_view()), name="account_email"),
        url(r"^confirm-email/$", EmailVerificationSentView.as_view(),
            name="account_email_verification_sent"),
        url(r"^confirm-email/(?P<key>[-:\w]+)/$", ConfirmEmailView.as_view(),
            name="account_confirm_email"),

        # password reset
        url(r"^password/reset/$", PasswordResetView.as_view(),
            name="account_reset_password"),
        url(r"^password/reset/done/$", PasswordResetDoneView.as_view(),
            name="account_reset_password_done"),
        url(r"^password/reset/key/(?P<uidb36>[0-9A-Za-z]+)/(?P<key>.+)/$",
            PasswordResetFromKeyView.as_view(),
            name="account_reset_password_from_key"),
        url(r"^password/reset/key/done/$", PasswordResetFromKeyDoneView.as_view(),
            name="account_reset_password_from_key_done"),
        # completely overrided
        # url(r'^$', include('allauth.urls')),


        # catch all
        # url(r'^(?P<path>.*)/$', WebpackBuiltTemplateView.as_view(), name='fallback'),
        # url(r'^(?P<path>.*)/$', RedirectView.as_view(url='/login/')),
    ]

if settings.DJANGO_SSO_APP_ENABLED:
    urlpatterns += [
        url(r'^$', TemplateView.as_view(template_name="home.html"), name='profile'),
    ]
sso_auth_urlpatterns = [
    url(r'^api/v1/groups/', include((groups_urls, 'groups'), namespace="group")),
]

if settings.DJANGO_SSO_BACKEND_ENABLED:
    from django_sso_app.core.apps.services.urls import urlpatterns as services_urls
    from django_sso_app.core.apps.passepartout.urls import urlpatterns as passepartout_urls
    from django_sso_app.core.apps.users.urls import urlpatterns as users_urls
    sso_auth_urlpatterns += [
        url(r'^api/v1/services/', include((services_urls, 'services'), namespace="service")),
        url(r'^api/v1/passepartout/', include((passepartout_urls, 'passepartout'), namespace="passepartout")),
        url(r'^api/v1/users/', include((users_urls, 'users'), namespace="user")),
    ]

if settings.DJANGO_SSO_APP_ENABLED:
    from django_sso_app.core.apps.profiles.urls import urlpatterns as profiles_urls
    sso_auth_urlpatterns += [
        url(r'^api/v1/profiles/', include((profiles_urls, 'profiles'), namespace="profile")),
    ]

urlpatterns += sso_auth_urlpatterns

# mine
if settings.DEBUG:
    urlpatterns = [
        url(r'^media/(?P<path>.*)$', serve,
            {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),
        ] + staticfiles_urlpatterns() + urlpatterns



import environ

env = environ.Env()

# django-sso
DJANGO_SSO_BACKEND_ENABLED = True
DJANGO_SSO_APP_ENABLED = env.bool('DJANGO_SSO_APP_ENABLED', default=(not DJANGO_SSO_BACKEND_ENABLED))

if DJANGO_SSO_BACKEND_ENABLED:
    from .backend import *

if DJANGO_SSO_APP_ENABLED:
    from .app import *



import logging

from allauth.account import app_settings as allauth_settings
from allauth.account.adapter import get_adapter
from allauth.account.models import EmailAddress
from allauth.utils import (get_username_max_length)

from rest_framework import serializers
from rest_framework.reverse import reverse

from django.conf import settings

from ...serializers import AbsoluteUrlSerializer
from ...functions import get_country_language

from .models import PROFILE_FIELDS, User

logger = logging.getLogger('users')


class CheckUserExistenceSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('sso_id',)
        read_only_fields = ('sso_id',)


class UserSerializer(serializers.ModelSerializer):
    subscriptions = serializers.SerializerMethodField(required=False)
    email_verified = serializers.SerializerMethodField(required=False)

    groups = serializers.SerializerMethodField(required=False)

    class Meta:
        model = User
        read_only_fields = (
        'sso_id', 'sso_rev', 'date_joined', 'username', 'is_unsubscribed',
        'is_active', 'subscriptions', 'email_verified', 'groups')
        fields = read_only_fields + PROFILE_FIELDS

    def get_subscriptions(self, instance):
        serialized = []
        for el in instance.subscriptions.all():
            serialized.append({
                'url': el.service.url,
                'is_unsubscribed': el.is_unsubscribed
            })
        return serialized

    def get_email_verified(self, instance):
        email_address = instance.emailaddress_set.get(email=instance.email)
        return email_address.verified

    def get_groups(self, instance):
        groups = list(instance.groups.values_list('name', flat=True))

        if instance.is_staff:
            groups.append('staff')

        return groups

    def update(self, instance, validated_data):
        user = instance
        request = self.context.get("request")
        requesting_user = request.user
        skip_confirmation = request.GET.get('skip_confirmation', None)
        must_confirm_email = True

        logger.info(
            'User {0} wants to update user {1}'.format(requesting_user, user))

        if skip_confirmation is not None and requesting_user.is_staff:
            must_confirm_email = False

        new_email = validated_data.get('email', None)
        if (new_email is not None):

            if new_email == user.email:  # !! because of app shitty logic (
                # always sends all fields)
                logger.info('{0} is has same email'.format(user))

            else:
                logger.info(
                    '{0} is changing email for user {1}, email confirmation '
                    'is {2}'.format(
                        requesting_user, user, must_confirm_email))

                adapter = get_adapter()

                created_email = EmailAddress.objects.add_email(request, user,
                                                               new_email,
                                                               confirm=must_confirm_email)
                logger.info(
                    'A new EmailAddress for {0} has been created with id {1}'
                    ''.format(
                        new_email, created_email.id))

                if not must_confirm_email:
                    adapter.confirm_email(None, created_email)

                setattr(user, '__email_updated', True)

        new_password = validated_data.get('password', None)
        if (new_password is not None):
            logger.info(
                '{0} is changing password for user {1}'.format(requesting_user,
                                                               user))
            validated_data.pop(
                'password')  # serializer must NOT save the received plain
            # password, password update logic performed by user.set_password

            user.set_password(new_password)

            setattr(user, '__password_updated', True)

        # devices will be removed by signal
        # actual_devices = user.devices.all()
        # actual_devices_count = actual_devices.count()
        # deleted_devices = user.remove_all_user_devices()

        return super(UserSerializer, self).update(instance, validated_data)

    def to_representation(self, obj):
        # get the original representation
        ret = super(UserSerializer, self).to_representation(obj)
        req = self.context.get('request', None)

        if req is not None:
            # remove field if password if not asked
            with_pass = req.query_params.get('with_password',
                                             not settings.DJANGO_SSO_HIDE_PASSWORD_FROM_USER_SERIALIZER)

            if not with_pass:
                ret.pop('password')

        # return the modified representation
        return ret


class NewUserSerializer(serializers.ModelSerializer):
    username = serializers.CharField(
        max_length=get_username_max_length(),
        min_length=allauth_settings.USERNAME_MIN_LENGTH,
        required=allauth_settings.USERNAME_REQUIRED
    )
    email = serializers.EmailField(required=True)

    referrer = serializers.CharField(required=True)

    class Meta:
        model = User
        read_only_fields = ('sso_id', 'sso_rev')
        fields = PROFILE_FIELDS + ('username', 'referrer') + read_only_fields

    def create(self, validated_data):
        vd = validated_data.copy()  # must remove referrer because
        # NewUserSerializer has no referrer field
        vd.pop('referrer', None)

        logger.debug('VALIDATED DATA NEW USER {}'.format(vd))
        user_language = vd.get('language', None)
        if user_language is None:
            user_country = vd.get('country')
            vd['language'] = get_country_language(user_country)

        # creating user from cleaned validated data
        new_user = super(NewUserSerializer, self).create(vd)

        user_email = validated_data.get('email')
        request = self.context.get("request")
        requesting_user = request.user
        skip_confirmation = request.query_params.get('skip_confirmation', None)
        must_confirm_email = True
        user_password = validated_data.get('password')
        password_is_hashed = request.query_params.get('password_is_hashed',
                                                      None)
        referrer = validated_data.get('referrer')

        if password_is_hashed is None:
            new_user.set_password(user_password)

            new_user.save()

        if skip_confirmation is not None and requesting_user != new_user and \
                requesting_user.is_staff:
            must_confirm_email = False

        email_address = EmailAddress.objects.add_email(request, new_user,
                                                       user_email,
                                                       confirm=must_confirm_email)

        if not must_confirm_email:
            email_address.verified = True
            email_address.set_as_primary(conditional=True)
            email_address.save()

        logger.info(
            '{0} has created user {1}'.format(requesting_user, new_user))

        new_user.subscribe_to_service(referrer)

        return new_user


class UserUnsubscriptionSerializer(serializers.Serializer):
    password = serializers.CharField()


class UserRevisionSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        read_only_fields = ('sso_id', 'sso_rev')
        fields = read_only_fields



import logging

logger = logging.getLogger('django_sso_app.core')


def get_country_language(country):
    logger.debug('Getting language for country {}'.format(country))
    if country == 'IT':
        return 'it'
    elif country == 'BR' or country == 'PT':
        return 'pt'
    elif country == 'ES':
        return 'es'

    return 'en-gb'



from django.contrib import admin

from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
# from django.contrib.auth.models import User

from .models import User
from .forms import UserChangeForm, UserCreationForm

from ...common import PROFILE_FIELDS

# Define a new User admin
class UserAdmin(BaseUserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm

    list_display = ('username', 'email', 'first_name', 'last_name', 'is_staff', 'last_login', 'role')
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups', 'last_login')

    fields = ('username', ) + PROFILE_FIELDS
    fieldsets = None

    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'email', 'password1', 'password2'),
        }),
    )

# Re-register UserAdmin
# admin.site.unregister(User)
admin.site.register(User, UserAdmin)



import logging

from rest_framework.reverse import reverse

from .models import Profile
from django_sso_app.core.serializers import AbsoluteUrlSerializer

logger = logging.getLogger('profiles')


class ProfileSerializer(AbsoluteUrlSerializer):

    class Meta:
        model = Profile
        read_only_fields = (
            'url',
            'created_at', 'sso_id', 'sso_rev', 'username', 'email',
            'first_name', 'last_name', 'description', 'birthdate', 'picture',
            'latitude', 'longitude', 'country', 'address')
        fields = read_only_fields

    def get_absolute_url(self, obj):
        if getattr(obj, 'id', None) is not None:
            request = self.context['request']
            reverse_url = reverse('profile:detail', args=[obj.sso_id])
            return request.build_absolute_uri(reverse_url)


class ProfilePublicSerializer(ProfileSerializer):

    class Meta:
        model = Profile
        read_only_fields = (
            'url',
            'sso_id',
            'created_at', 'username', 'picture',
            'latitude', 'longitude', 'country')
        fields = read_only_fields



import logging

from django.utils.translation import activate as activate_translation
from django.conf import settings
from django.urls import reverse

from allauth.account.adapter import DefaultAccountAdapter

from django_sso_app.core.common import PROFILE_FIELDS

logger = logging.getLogger('backend')


class UserAdapter(DefaultAccountAdapter):

    def save_user(self, request, user, form, commit=True):
        """
        Saves a new `User` instance using information provided in the
        signup form.
        """

        cleaned_data = form.get_cleaned_data()
        logger.info('Saving new user {0} with email {1}'.format(cleaned_data.get('username'), cleaned_data.get('email')))

        for f in PROFILE_FIELDS:
            setattr(user, f, cleaned_data.get(f, None))

        new_user = super(UserAdapter, self).save_user(
            request, user, form, commit)

        return new_user

    def render_mail(self, template_prefix, email, context):
        user = context.get('user')
        user_language = user.language
        logger.info('Rendering mail for {0} with language {1}'.format(user, user_language))
        activate_translation(user_language)
        context.update({
            'EMAILS_DOMAIN': settings.EMAILS_DOMAIN,
            'EMAILS_SITE_NAME': settings.EMAILS_SITE_NAME
            })
        return super(UserAdapter, self).render_mail(template_prefix, email, context)

    def get_email_confirmation_url(self, request, emailconfirmation):
        """Constructs the email confirmation (activation) url.
        Note that if you have architected your system such that email
        confirmations are sent outside of the request context `request`
        can be `None` here.
        """
        url = reverse(
            "account_confirm_email",
            args=[emailconfirmation.key])
        ret = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL + "://" + settings.EMAILS_DOMAIN + url
        return ret




import environ

env = environ.Env()

from .settings import * # django defaults
from .logging import *

from django_sso_app.core.settings import *

gettext = lambda s: s

TESTING_MODE = 'test' in sys.argv or DEBUG

SITE_ID = 1

LANGUAGES = (
    ## Customize this
    ('it', gettext('it')),
    ('en-gb', gettext('en')),
    ('es', gettext('es')),
    ('pt', gettext('pt')),
    ('fr', gettext('fr')),
    ('de', gettext('de')),
)

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'core', 'locale'),
)

LOGIN_URL = '/login/'
LOGIN_REDIRECT_URL = '/'

APP_DOMAIN = os.environ.get('APP_DOMAIN', 'localhost:8000')
APP_URL = ACCOUNT_DEFAULT_HTTP_PROTOCOL + '://' + APP_DOMAIN

# email
DEFAULT_FROM_EMAIL = os.environ.get('DJANGO_DEFAULT_FROM_EMAIL', 'noreply@example.com')

# sendgrid
EMAIL_BACKEND = "sendgrid_backend.SendgridBackend"
SENDGRID_API_KEY=os.environ.get("SENDGRID_API_KEY", "")
#SENDGRID_SANDBOX_MODE_IN_DEBUG=False # defaults to True

if DEBUG or os.environ.get('DJANGO_CONSOLE_EMAIL_BACKEND', None):
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'


EMAILS_DOMAIN = os.environ.get('EMAILS_DOMAIN', APP_DOMAIN) # domain name specified in email templates
EMAILS_SITE_NAME = os.environ.get('EMAILS_SITE_NAME', EMAILS_DOMAIN) # site name specified in email templates

COOKIE_DOMAIN = "example.com"


INSTALLED_APPS += [
    'rest_framework',
    'rest_framework.authtoken',
    'rest_auth',

    'allauth',
    'allauth.account',
    'allauth.socialaccount',

    'rest_auth.registration',
    'rest_framework_swagger',

    'corsheaders',
] + DJANGO_SSO_BACKEND_CORE_APPS


if DJANGO_SSO_APIGW_ENABLED:
    INSTALLED_APPS += [
        'django_sso_app.core.apps.api_gateway.kong',
    ]

AUTH_USER_MODEL = 'users.User'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'core.db.sqlite3'),
    }
}

# djangorestframework

DRF_DEFAULT_AUTHENTICATION_CLASSES = [
    'rest_framework.authentication.TokenAuthentication'
]

if DEBUG:
    DRF_DEFAULT_AUTHENTICATION_CLASSES = ['rest_framework.authentication.SessionAuthentication'] + DRF_DEFAULT_AUTHENTICATION_CLASSES

DRF_DEFAULT_AUTHENTICATION_CLASSES = DRF_DEFAULT_AUTHENTICATION_CLASSES + [
    'django_sso_app.core.apps.users.authentication.UsersAuthentication',
]

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': DRF_DEFAULT_AUTHENTICATION_CLASSES,
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'PAGE_SIZE': 100,
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),

    'DATETIME_FORMAT': '%Y-%m-%dT%H:%M:%S%z',
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    # https://www.django-rest-framework.org/community/3.10-announcement/#continuing-to-use-coreapi
}


# allauth
AUTHENTICATION_BACKENDS = [
    # Needed to login by username in Django admin, regardless of `allauth`
    'django.contrib.auth.backends.ModelBackend',
    # `allauth` specific authentication methods, such as login by e-mail
    'allauth.account.auth_backends.AuthenticationBackend',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',

    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.locale.LocaleMiddleware',

    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',

    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


ACCOUNT_DEFAULT_HTTP_PROTOCOL = 'http'

ACCOUNT_AUTHENTICATION_METHOD='username_email'
ACCOUNT_EMAIL_REQUIRED=True
ACCOUNT_USERNAME_REQUIRED = True
ACCOUNT_ADAPTER='django_sso_app.backend.adapter.UserAdapter'
ACCOUNT_CONFIRM_EMAIL_ON_GET=True
ACCOUNT_EMAIL_VERIFICATION='mandatory'
ACCOUNT_AUTHENTICATED_LOGIN_REDIRECTS=False
#ACCOUNT_SESSION_REMEMBER=False
#AUTHENTICATED_LOGIN_REDIRECTS=True
ACCOUNT_EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL = LOGIN_URL
ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL = LOGIN_URL
ACCOUNT_SESSION_REMEMBER=False

ACCOUNT_FORMS = {
    'signup': 'django_sso_app.backend.forms.SignupForm'
}


# rest_auth

REST_USE_JWT = True
REST_AUTH_SERIALIZERS = {
    'LOGIN_SERIALIZER': 'django_sso_app.backend.serializers.LoginSerializer',
    'PASSWORD_RESET_SERIALIZER': 'django_sso_app.backend.serializers.PasswordResetSerializer',
    'USER_DETAILS_SERIALIZER' : 'django_sso_app.core.apps.users.serializers.UserSerializer',
}

REST_AUTH_REGISTER_SERIALIZERS = {
    'REGISTER_SERIALIZER': 'django_sso_app.backend.serializers.RegisterSerializer',
}

JWT_AUTH = {
    'JWT_AUTH_COOKIE': 'jwt',

    'JWT_PAYLOAD_HANDLER': 'django_sso_app.backend.handlers.jwt_payload_handler',
    'JWT_AUTH_HEADER_PREFIX': 'Bearer',
    'JWT_VERIFY': False,
}






import logging

from rest_framework import permissions
from rest_framework import viewsets

from django_sso_app.core.permissions import is_staff
from .models import Profile
from .serializers import ProfileSerializer, ProfilePublicSerializer

logger = logging.getLogger('profiles')


class ProfileViewSet(viewsets.ReadOnlyModelViewSet):
    lookup_field = 'sso_id'

    def get_queryset(self):
        user = self.request.user
        if is_staff(user):
            return Profile.objects.all()
        else:
            return Profile.objects.filter(user=user)

    def get_serializer_class(self):
        if self.request.user.is_authenticated:
            return ProfileSerializer
        else:
            return ProfilePublicSerializer

    def list(self, request, *args, **kwargs):
        """
        List profiles
        """
        return super(ProfileViewSet, self).list(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        """
        Profile detail
        """
        return super(ProfileViewSet, self).retrieve(request, *args, **kwargs)



ROLE_CHOICES = (
    (-1, 'Staff'),
    (0, 'Cityzen'),
    (1, 'Institutional'),
    (2, 'Professionist'),
    (4, 'Company'),
    (6, 'Istitutional Partition Item'),
    (7, 'App'),
)

PROFILE_FIELDS = (
    'email',
    'password',
    'role',
    'first_name', 'last_name',
    'description', 'picture', 'birthdate',
    'latitude', 'longitude',
    'country',
    'address',
    'language'
)
